"use strict";

let THREECAMERA;
var SETTINGS = {
    gltfModelURL: 'models3D/Jendy-updated.gltf',
    cubeMapURL: 'Bridge2/',
    rotationOffsetX: 5, //negative -> look upper. in radians
    rotationOffsetY: 600, //negative -> look upper. in radians
    cameraFOV: 40,      //in degrees, 3D camera FOV
    pivotOffsetYZ: [0, 0], //XYZ of the distance between the center of the cube and the pivot
    detectionThreshold: 0.75, //sensibility, between 0 and 1. Less -> more sensitive
    detectionHysteresis: 0.05,
    offsetYZ: [0, -0.4], //offset of the model in 3D along vertical and depth axis
    scale: 1.1, //width in 3D of the GLTF model
};

// callback : launched if a face is detected or lost. TODO : add a cool particle effect WoW !
function detect_callback(faceIndex, isDetected) {
    if (isDetected) {
        console.log('INFO in detect_callback() : DETECTED');
    } else {
        console.log('INFO in detect_callback() : LOST');
    }
}

// build the 3D. called once when Jeeliz Face Filter is OK
function init_threeScene(spec) {
    const threeStuffs = THREE.JeelizHelper.init(spec, detect_callback);

    // CREATE THE GLASSES AND ADD THEM
    const r = JeelizThreeGlassesCreator({
        envMapURL: "envMap.jpg",
        frameMeshURL: "models3D/glassesFramesBranchesBent.json",
        lensesMeshURL: "models3D/glassesLenses.json",
        occluderURL: "models3D/face.json"
    });

    threeStuffs.faceObject.add(r.occluder);
    //r.occluder.rotation.set(-0.15,0,0);
    window.re=r.occluder;
    r.occluder.rotation.set(0.3,0,0);
    r.occluder.position.set(0,0.1,-0.04);
    r.occluder.scale.multiplyScalar(0.0085);

    // const threeGlasses = r.glasses;
    // //threeGlasses.rotation.set(-0.15,0,0); //X neg -> rotate branches down
    // threeGlasses.position.set(0, 0.07, 0.4);
    // threeGlasses.scale.multiplyScalar(0.006);
    // threeStuffs.faceObject.add(threeGlasses);


    var path = SETTINGS.cubeMapURL;
    var format = '.jpg';
    var envMap = new THREE.CubeTextureLoader().load([
        path + 'posx' + format, path + 'negx' + format,
        path + 'posy' + format, path + 'negy' + format,
        path + 'posz' + format, path + 'negz' + format
    ]);

    //IMPORT THE GLTF MODEL
    //from https://threejs.org/examples/#webgl_loader_gltf
    var gltfLoader = new THREE.GLTFLoader();
    gltfLoader.load(SETTINGS.gltfModelURL, function (gltf) {
        gltf.scene.traverse(function (child) {
            // console.log(child);
            if (child.isMesh) {
                // console.log(child);
                // var uniform = child.material.uniforms;
                // uniforms.uBranchFading = new THREE.Vector2(-90, 60);
                //custom material with fading at the end of the branches
                const uniforms = Object.assign(THREE.ShaderLib.basic.uniforms,
                    {
                        envMap: {value: envMap},
                        color: {value: new THREE.Color(0xf0f0f0)},
                        uBranchFading: {value: new THREE.Vector2(-90, 60)} //first value: position (lower -> to the back), second: transition brutality
                    });

                //tweak vertex shader to give the Z of the current point
                var vertexShaderSource = "varying float vPosZ;\n" + THREE.ShaderLib.basic.vertexShader;
                vertexShaderSource = vertexShaderSource.replace('#include <fog_vertex>', 'vPosZ=position.z;');

                //tweak fragment shader to apply transparency at the
                // end of the branches
                var fragmentShaderSource = "uniform vec2 uBranchFading;\n varying float vPosZ;\n" + THREE.ShaderLib.basic.fragmentShader;
                const GLSLcomputeAlpha = 'gl_FragColor.a=smoothstep(uBranchFading.x-uBranchFading.y*0.5, uBranchFading.x+uBranchFading.y*0.5, vPosZ);'
                fragmentShaderSource = fragmentShaderSource.replace('#include <fog_fragment>', GLSLcomputeAlpha);

                // uniforms. = child.material;
                var mat =
                    new THREE.ShaderMaterial({
                        uniforms: uniforms,
                        vertexShader: vertexShaderSource,
                        fragmentShader: fragmentShaderSource
                    });

                // console.log(child.material)


                // child.material.map = mat;


            }
        });
        // gltf.scene.frustumCulled = false;


        //center and scale the object
        var bbox = new THREE.Box3().expandByObject(gltf.scene);

        //center the model :
        var centerBBox = bbox.getCenter(new THREE.Vector3());
        gltf.scene.position.add(centerBBox.multiplyScalar(-1));
        gltf.scene.position.add(new THREE.Vector3(0, 0.38, 0.58));
        gltf.scene.rotation.set(SETTINGS.rotationOffsetX - 0.2, SETTINGS.rotationOffsetY + 0.005, 0, "XYZ");

        //scale the model according to its width
        var sizeX = bbox.getSize(new THREE.Vector3()).x;
        gltf.scene.scale.multiplyScalar(SETTINGS.scale / sizeX);
        var node_r = gltf.scene.getObjectByName('Node_Cut-Extrude18');
        node_r.rotation.x = -0.12;
        node_r.rotation.y = -0.15;
        node_r.rotation.z = 0;
        // console.log(node_r.rotation);
        var node_l = gltf.scene.getObjectByName('Node_Cut-Extrude19');
        node_l.rotation.x = -0.12;
        node_l.rotation.y = 0.15;
        node_l.rotation.z = 0;
        // console.log(node_l.rotation);

        threeStuffs.faceObject.add(gltf.scene);



        const color = 0xFFFFFF;
        const intensity = 2;
        const light = new THREE.AmbientLight(color, intensity);
        threeStuffs.faceObject.add(light);
    });


    //CREATE THE CAMERA
    const aspecRatio = spec.canvasElement.width / spec.canvasElement.height;
    THREECAMERA = new THREE.PerspectiveCamera(40, aspecRatio, 0.1, 100);

} // end init_threeScene()

var canvId='';

//launched by body.onload() :
function main(canvasId) {
	canvId=canvasId;
    JeelizResizer.size_canvas({
        canvasId: canvasId,
        callback: function (isError, bestVideoSettings) {
            init_faceFilter(bestVideoSettings);
        }
    })
} //end main()

function init_faceFilter(videoSettings) {
    JEEFACEFILTERAPI.init({
        followZRot: true,
        canvasId: canvId,
        NNCpath: './jeeliz/dist/', // root of NNC.json file
        maxFacesDetected: 1,
        callbackReady: function (errCode, spec) {
            if (errCode) {
                console.log('AN ERROR HAPPENS. ERR =', errCode);
                return;
            }

            console.log('INFO : JEEFACEFILTERAPI IS READY');
            init_threeScene(spec);
        }, //end callbackReady()

        //called at each render iteration (drawing loop) :
        callbackTrack: function (detectState) {
            THREE.JeelizHelper.render(detectState, THREECAMERA);
        } //end callbackTrack()
    }); //end JEEFACEFILTERAPI.init call
} // end main()

